import 'package:bolter_flutter/src/persist/persist.dart';
import 'package:flutter/material.dart';

class PersistLifecycleWrapper<DS, FS> extends StatefulWidget {
  final Widget child;
  final Persist<DS, FS> persist;
  final int version;
  final DS domainState;
  final FS flutterState;

  const PersistLifecycleWrapper({
    Key key,
    @required this.child,
    @required this.persist,
    this.version,
    this.domainState,
    this.flutterState,
  }) : super(key: key);

  @override
  _PersistLifecycleWrapperState createState() =>
      _PersistLifecycleWrapperState();
}

class _PersistLifecycleWrapperState extends State<PersistLifecycleWrapper>
    with WidgetsBindingObserver {
  @override
  void initState() {
    WidgetsBinding.instance.addObserver(this);
    super.initState();
  }

  @override
  Widget build(BuildContext context) => widget.child;

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.paused ||
        state == AppLifecycleState.inactive) {
      widget.persist.save(
          domainState: widget.domainState,
          flutterState: widget.flutterState,
          version: widget.version);
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }
}
